import QtQuick 1.1
import com.meego 1.0
import QtWebKit 1.0

Rectangle {

    id:webContainer

    property string url
    property bool isSpinning: false
    property bool backButtonShouldShow: true
    signal connectionNotAvailable();

    function openUrl(anUrl, forceExternal,backButtonEnabled) {

        if(forceExternal == false) {

            // first make sure everything is visible,
            // including spinner
            webContainer.visible = true
            webContainer.showSpinning()
            // set URL somewhere accessible
            webContainer.url = anUrl
            // dynamically load a web view
            webViewLoader.sourceComponent = webViewComponent
            backButtonShouldShow = backButtonEnabled

        } else {

            // for whatever reason, use the full native browser
            Qt.openUrlExternally(anUrl)
        }
    }

    function showSpinning() {

        if(!isSpinning) {

            //console.log("show loading")
            appState.loading = true
            webViewSpinner.opacity = 1
            webViewSpinner.visible = true
            webViewSpinner.running = true
            isSpinning = true
        }
    }

    function stopSpinning() {

        if(isSpinning) {

            //console.log("stop loading")
            appState.loading = false
            webViewSpinner.opacity = 0
            webViewSpinner.visible = false
            webViewSpinner.running = false
            isSpinning = false
        }
    }

    // hook back button to this function
    // because web view is dynamically loaded
    // and thus not directly accessible
    function goBack() {

        // loader is done loading
        if(webViewLoader.status == Loader.Ready) {

            //webContainer.stopSpinning()

            // web view's back action exposed
            // on loader's item which is flickable
            webViewLoader.item.backAction.trigger()
        }
    }

    function closeWebView() {

        webContainer.stopSpinning()
        webContainer.visible = false
        // completely unload dynamic web view
        webViewLoader.sourceComponent = undefined
    }

    visible: false
    color: "white"

    BusyIndicator {

        id: webViewSpinner

        anchors.centerIn: webContainer
        // Use opacity to hide/show the indicator. Also animate the enterance/exit
        // by using Behaviour on opacity.
        opacity: 1
        running: visible
        Behavior on opacity { PropertyAnimation { duration: 250 } }
        platformStyle: BusyIndicatorStyle { size: "large" }
        z: 701
    }

    // web view loaded for new URLs
    // main motivation is for new back stack/history
    Loader {

        id: webViewLoader

        anchors.fill: webContainer
    }

    Component {

        id: webViewComponent

        Flickable {

            id: flickable

            // external hook for firing back action
            property alias backAction: webView.back


            contentWidth: Math.max(flickable.width,webView.width)
            contentHeight: Math.max(flickable.height,webView.height)
            pressDelay: 100
            clip: true

            WebView {
                id: webView

                // binding to web container's url
                // which should be set before we loaded
                url: webContainer.url

                transformOrigin: Item.TopLeft
                smooth: false // We don't want smooth scaling, since we only scale during (fast) transitions
                preferredWidth: flickable.width
                preferredHeight: flickable.height
                contentsScale: 1
                settings.privateBrowsingEnabled:true
                settings.defaultFixedFontSize:20

                onLoadStarted: {

                    //console.log("on load started")
                    webContainer.showSpinning()
                }

                onLoadFinished: {

                    //console.log("on load finished")
                    webContainer.stopSpinning()

                    // successfully done loading
                    // so setup the back button
                    backButton.visible = back.enabled && backButtonShouldShow == true
                }

                onLoadFailed: {

                    webContainer.closeWebView()
                    webContainer.connectionNotAvailable()
                }
            }
        }
    }

    Button {
        id:buttonExit
        height:47
        width:47
        z:320
        anchors.top: parent.top
        anchors.right: parent.right
        onClicked: webContainer.closeWebView()

        Image {
            id: buttonWebExit
            source: props.theme.images.webViewExit
        }
    }

    Button {
        id: backButton

        width: 47
        height: 47
        z: 320
        // start out invisible
        visible: false
        anchors {
            top: parent.top
            left: parent.left
        }
        // go back function hooks into dynamic web view
        onClicked: goBack()
        Image {
            id: backButtonImage
            source: props.theme.images.webViewBack
        }
    }
}
